@@ -29,7 +29,7 @@ from pypeit.core import moment, pydl, arc
 # TODO: Some of these functions could probably just live in pypeit.edges
 
 def detect_slit_edges(flux, bpm=None, median_iterations=0, min_sqm=30., sobel_mode='nearest',
-                      sigdetect=30., grow_bpm=5):
+                      sobel_enhance=0, sigdetect=30., grow_bpm=5):
     r"""
     Find slit edges using the input image.
 
@@ -60,17 +60,20 @@ def detect_slit_edges(flux, bpm=None, median_iterations=0, min_sqm=30., sobel_mo
         sobel_mode (:obj:`str`, optional):
             Mode to use with the Sobel filter.  See
             `scipy.ndimage.sobel`_.
+        sobel_enhance (:obj:`int`, optional):
+            If slit edges are not well-defined (e.g. blurred) set this
+            parameter to a number greater than zero to enhance the edge detection.
         sigdetect (:obj:`float`, optional):
             Threshold for edge detection.
-        grow_bpm (:int)
+        grow_bpm (int, optional):
             The sobel_sig and edg_img are masked using the bpm. This is done by convolving the bpm with
             a spatial boxcar of width grow_bpm pixels, ensuring that pixels which touched bad pixels are also masked.
 
     Returns:
-        Returns two `numpy.ndarray`_ objects: (1) The image of the
-        significance of the edge detection in sigma and (2) the array
-        isolating the slit edges. In the latter, left edges have a
-        value of -1 and right edges have a value of 1.
+        tuple: Returns two `numpy.ndarray`_ objects -- (1) The image of the
+            significance of the edge detection in sigma and (2) the array
+            isolating the slit edges. In the latter, left edges have a
+            value of -1 and right edges have a value of 1.
     """
     # Checks
     if flux.ndim != 2:
@@ -97,6 +100,9 @@ def detect_slit_edges(flux, bpm=None, median_iterations=0, min_sqm=30., sobel_mo
 
     # Filter with a Sobel
     filt = ndimage.sobel(sqmstrace, axis=1, mode=sobel_mode)
+    # Enhance blurred edges
+    if sobel_enhance > 0:
+        filt = sobel_enhance * ndimage.uniform_filter1d(filt, size=sobel_enhance, axis=1)
     # Apply the bad-pixel mask
     if bpm is not None:
         # NOTE: Casts to float because filt is float
@@ -108,7 +114,7 @@ def detect_slit_edges(flux, bpm=None, median_iterations=0, min_sqm=30., sobel_mo
     # TODO: why not match the sign of the Sobel image to the edge it
     # traces? I.e., why is the sign flipped?
     # Answer: I defined left as -1 (i.e. counting from left to right (-1, 0, +1) = (left, middle, right)
-    tedges = np.zeros(flux.shape, dtype=np.float)
+    tedges = np.zeros(flux.shape, dtype=float)
     tedges[np.where(sobel_sig > sigdetect)] = -1.0  # A positive gradient is a left edge
     tedges[np.where(sobel_sig < -sigdetect)] = 1.0  # A negative gradient is a right edge
 
@@ -116,7 +122,7 @@ def detect_slit_edges(flux, bpm=None, median_iterations=0, min_sqm=30., sobel_mo
     # Clean the edges
     wcl = np.where((ndimage.maximum_filter1d(sobel_sig, 10, axis=1) == sobel_sig) & (tedges == -1))
     wcr = np.where((ndimage.minimum_filter1d(sobel_sig, 10, axis=1) == sobel_sig) & (tedges == 1))
-    edge_img = np.zeros(sobel_sig.shape, dtype=np.int)
+    edge_img = np.zeros(sobel_sig.shape, dtype=int)
     edge_img[wcl] = -1
     edge_img[wcr] = 1
 
@@ -265,8 +271,9 @@ def count_edge_traces(edge_img):
             and negative numbers follow left slit edges.
     
     Returns:
+        int or tuple:
         Two integers with the number of left and right edges,
-        respectively.
+        respectively.  Or 0 if the minimum input value is 0
     """
     # Avoid returning -0
     nleft = np.amin(edge_img)
@@ -476,25 +483,27 @@ def prepare_sobel_for_trace(sobel_sig, bpm=None, boxcar=5, side='left'):
 
     This method:
         - Flips and/or truncates the pixel values based on the edge
-          side to be traced (see `side`).
+          side to be traced (see ``side``).
         - Smooths along rows (spatially)
 
     Args:           
         sobel_sig (`numpy.ndarray`_):
             Image with the significance of the edge detection; see
             :func:`detect_slit_edges`.
+        bpm (`numpy.ndarray`_, optional):
+            Boolean bad-pixel mask (bad pixels are True).  If None, all pixels
+            are considered good.  Must have the same shape as ``sobel_sig``.
         boxcar (:obj:`int`, optional):
-            Boxcar smooth the detection image along rows before
-            recentering the edge centers; see
-            :func:`pypeit.utils.boxcar_smooth_rows`. If `boxcar` is
-            less than 1, no smoothing is performed.
+            Boxcar smooth the detection image along rows before recentering the
+            edge centers; see :func:`~pypeit.utils.boxcar_smooth_rows`. If less
+            than 1, no smoothing is performed.
         side (:obj:`str`, optional):
-            The side that the image will be used to trace. In the
-            Sobel image, positive values are for left traces,
-            negative for right traces. If 'left', the image is
-            clipped at a minimum value of -0.1. If 'right', the image
-            sign is flipped and then clipped at a minimum of -0.1. If
-            None, the image is not flipped or clipped, only smoothed.
+            The side that the image will be used to trace. In the Sobel image,
+            positive values are for left traces, negative for right traces. If
+            ``'left'``, the image is clipped at a minimum value of -0.1. If
+            ``'right'``, the image sign is flipped and then clipped at a minimum
+            of -0.1. If None, the image is not flipped or clipped, only
+            smoothed.
 
     Returns:
         `numpy.ndarray`_: The smoothed image.
@@ -508,7 +517,7 @@ def prepare_sobel_for_trace(sobel_sig, bpm=None, boxcar=5, side='left'):
     # TODO: This 0.1 is drawn out of the ether and different from what is done in peak_trace
     img = sobel_sig if side is None else np.maximum((1 if side == 'left' else -1)*sobel_sig, 0.1)
     # Returned the smoothed image
-    wgt = None if bpm is None else np.invert(bpm)
+    wgt = None if bpm is None else np.logical_not(bpm).astype(float)
     return utils.boxcar_smooth_rows(img, boxcar, wgt=wgt, replace='zero')
 
 
@@ -599,9 +608,9 @@ def follow_centroid(flux, start_row, start_cen, ivar=None, bpm=None, fwgt=None,
             uses the DISCONTINUOUS flag.
 
     Returns:
-        Three numpy arrays are returned: the optimized center, an
-        estimate of the error, and a bad-pixel mask (masked values
-        are True).
+        tuple: Three numpy arrays are returned. the optimized center, an
+            estimate of the error, and a bad-pixel mask (masked values
+            are True).
     """
     if flux.ndim != 2:
         raise ValueError('Input image must be 2D.')
@@ -772,9 +781,9 @@ def masked_centroid(flux, cen, width, ivar=None, bpm=None, fwgt=None, row=None,
             dummy value.
 
     Returns:
-        Returns three `numpy.ndarray`_ objects: the new centers, the
-        center errors, and the measurement flags with a data type
-        depending on `bitmask`.
+        tuple: Returns three `numpy.ndarray`_ objects. the new centers, the
+            center errors, and the measurement flags with a data type
+            depending on `bitmask`.
     """
     # Calculate the moments
     radius = width/2
@@ -1529,7 +1538,7 @@ def peak_trace(flux, ivar=None, bpm=None, trace_map=None, extract_width=None, sm
 
 def parse_user_slits(add_slits, this_det, rm=False):
     """
-    Parse the parset syntax for adding slits
+    Parse the parset syntax for adding or removing slits
 
     Args:
         add_slits (str, list):
@@ -1541,8 +1550,8 @@ def parse_user_slits(add_slits, this_det, rm=False):
 
     Returns:
         list or None:
-          if list,  [[x0,x1,yrow]] for add with one or more entries
-          if list,  [[xcen,yrow]] for rm with one or more entries
+          if list,  [[x0,x1,yrow]] for add (rm=False) with one or more entries
+          if list,  [[xcen,yrow]] for rm=True with one or more entries
 
     """
     # Might not be a list yet (only a str)
@@ -1551,11 +1560,12 @@ def parse_user_slits(add_slits, this_det, rm=False):
     #
     user_slits = []
     for islit in add_slits:
+        # Add?
         if not rm:
             det, x0, x1, yrow = [int(ii) for ii in islit.split(':')]
             if det == this_det:
                 user_slits.append([x0,x1,yrow])
-        else:
+        else: # Remove
             det, xcen, yrow = [int(ii) for ii in islit.split(':')]
             if det == this_det:
                 user_slits.append([xcen,yrow])
