@@ -3,7 +3,7 @@
 import numpy as np
 from scipy.ndimage.filters import maximum_filter
 from scipy.ndimage.morphology import generate_binary_structure, binary_erosion
-import numba as nb
+#import numba as nb
 
 
 def detect_2Dpeaks(image):
@@ -264,7 +264,7 @@ def score_quad_matches(fidx):
     return scores
 
 
-@nb.jit(nopython=True, cache=True)
+#@nb.jit(nopython=True, cache=True)
 def triangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
     """
 
@@ -338,8 +338,8 @@ def triangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
         else:
             cntlst += lup
 
-    lindex = np.zeros((cntdet*cntlst, nptn), dtype=nb.types.uint64)
-    dindex = np.zeros((cntdet*cntlst, nptn), dtype=nb.types.uint64)
+    lindex = np.zeros((cntdet*cntlst, nptn),dtype=np.uint64) #, dtype=nb.types.uint64)
+    dindex = np.zeros((cntdet*cntlst, nptn),dtype=np.uint64) #, dtype=nb.types.uint64)
     wvcen = np.zeros((cntdet*cntlst))
     disps = np.zeros((cntdet*cntlst))
 
@@ -383,7 +383,7 @@ def triangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
     return dindex, lindex, wvcen, disps
 
 
-@nb.jit(nopython=True, cache=True)
+#@nb.jit(nopython=True, cache=True)
 def quadrangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
     """
 
@@ -437,10 +437,10 @@ def quadrangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
     sz_d = detlines.size
     sz_l = linelist.size
 
-    lindex = np.zeros((1, nptn), dtype=nb.types.uint64)
-    dindex = np.zeros((1, nptn), dtype=nb.types.uint64)
-    wvcen = np.zeros((1,), dtype=nb.types.ulong)
-    disps = np.zeros((1,), dtype=nb.types.ulong)
+    lindex = np.zeros((1, nptn),dtype=np.uint64) #, dtype=nb.types.uint64)
+    dindex = np.zeros((1, nptn),dtype=np.uint64) #, dtype=nb.types.uint64)
+    wvcen = np.zeros((1,), dtype=np.ulonglong) #, dtype=nb.types.ulong)
+    disps = np.zeros((1,), dtype=np.ulonglong) #, dtype=nb.types.ulong)
 
     # Generate the patterns
     for dl in range(0, sz_d-nptn+1):  # dl is the starting point of the detlines pattern
@@ -480,17 +480,17 @@ def quadrangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
                                             tst *= -1.0
                                         if tst <= tol:
                                             # The second pattern matches, store the result!
-                                            lindex = np.vstack((lindex, np.array([[ll, la, lb, lr]], dtype=nb.types.uint64)))
-                                            dindex = np.vstack((dindex, np.array([[dl, da, db, dr]], dtype=nb.types.uint64)))
+                                            lindex = np.vstack((lindex, np.array([[ll, la, lb, lr]], dtype=np.uint64))) #, dtype=nb.types.uint64)))
+                                            dindex = np.vstack((dindex, np.array([[dl, da, db, dr]], dtype=np.uint64))) #, dtype=nb.types.uint64)))
                                             tst = (linelist[lr] - linelist[ll]) / (detlines[dr] - detlines[dl])
                                             wvl = (npixels/2.)*tst + (linelist[lr]-tst*detlines[dr])
-                                            wvcen = np.hstack((wvcen, np.array([wvl], dtype=nb.types.ulong)))
-                                            disps = np.hstack((disps, np.array([tst], dtype=nb.types.ulong)))
+                                            wvcen = np.hstack((wvcen, np.array([wvl], dtype=np.ulonglong))) #, dtype=nb.types.ulong)))
+                                            disps = np.hstack((disps, np.array([tst], dtype=np.ulonglong))) # , dtype=nb.types.ulong)))
     # Return, but first remove the spurious first entry due to array creation
     return dindex[1:, :], lindex[1:, :], wvcen[1:], disps[1:]
 
 
-@nb.jit(nopython=True, cache=True)
+#@nb.jit(nopython=True, cache=True)
 def curved_quadrangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixtol=1.0):
     """
 
@@ -545,10 +545,10 @@ def curved_quadrangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixto
     sz_d = detlines.size
     sz_l = linelist.size
 
-    lindex = np.zeros((1, nptn), dtype=nb.types.uint64)
-    dindex = np.zeros((1, nptn), dtype=nb.types.uint64)
-    wvcen = np.zeros((1,), dtype=nb.types.ulong)
-    disps = np.zeros((1,), dtype=nb.types.ulong)
+    lindex = np.zeros((1, nptn), dtype=np.uint64) #, dtype=nb.types.uint64)
+    dindex = np.zeros((1, nptn), dtype=np.uint64) #, dtype=nb.types.uint64)
+    wvcen = np.zeros((1,), dtype=np.ulonglong) #, dtype=nb.types.ulong)
+    disps = np.zeros((1,), dtype=np.ulonglong) #, dtype=nb.types.ulong)
 
     # Generate the patterns
     for dl in range(0, sz_d-nptn+1):  # dl is the starting point of the detlines pattern
@@ -590,12 +590,12 @@ def curved_quadrangles(detlines, linelist, npixels, detsrch=5, lstsrch=10, pixto
                                             tst *= -1.0
                                         if tst <= tol:
                                             # The second pattern matches, store the result!
-                                            lindex = np.vstack((lindex, np.array([[ll, la, lb, lr]], dtype=nb.types.uint64)))
-                                            dindex = np.vstack((dindex, np.array([[dl, dm, da, dr]], dtype=nb.types.uint64)))
+                                            lindex = np.vstack((lindex, np.array([[ll, la, lb, lr]], dtype=np.uint64)))#, dtype=nb.types.uint64)))
+                                            dindex = np.vstack((dindex, np.array([[dl, dm, da, dr]], dtype=np.uint64)))#, dtype=nb.types.uint64)))
                                             tst = (linelist[lr] - linelist[ll]) / (detlines[dr] - detlines[dl])
                                             wvl = (npixels/2.)*tst + (linelist[lr]-tst*detlines[dr])
-                                            wvcen = np.hstack((wvcen, np.array([wvl], dtype=nb.types.ulong)))
-                                            disps = np.hstack((disps, np.array([tst], dtype=nb.types.ulong)))
+                                            wvcen = np.hstack((wvcen, np.array([wvl], dtype=np.ulonglong))) #, dtype=nb.types.ulong)))
+                                            disps = np.hstack((disps, np.array([tst], dtype=np.ulonglong))) #, dtype=nb.types.ulong)))
     # Return, but first remove the spurious first entry due to array creation
     return dindex[1:, :], lindex[1:, :], wvcen[1:], disps[1:]
 
@@ -614,7 +614,7 @@ def empty_patt_dict(nlines):
         An empty pattern dictionary
 
     """
-    patt_dict = dict(acceptable=False, nmatch=0, ibest=-1, bwv=0., sign=1, mask=np.zeros(nlines, dtype=np.bool))
+    patt_dict = dict(acceptable=False, nmatch=0, ibest=-1, bwv=0., sign=1, mask=np.zeros(nlines, dtype=bool))
     return patt_dict
 
 
@@ -655,7 +655,7 @@ def solve_xcorr(detlines, linelist, dindex, lindex, line_cc, nreid_min = 4, cc_l
     cc_avg = np.zeros(nlines)
     scores =np.zeros(nlines,dtype='<U50')
     scores[:] = 'None'
-    mask = np.zeros(nlines, dtype=np.bool)
+    mask = np.zeros(nlines, dtype=bool)
     ngd_match = 0
     for dd in range(nlines):
         # Grab all the instances of this detected line's pixel position index
@@ -668,7 +668,7 @@ def solve_xcorr(detlines, linelist, dindex, lindex, line_cc, nreid_min = 4, cc_l
         cc_per_match = np.zeros(unq.size,dtype=float)
         for iuniq, unq_val in enumerate(unq):
             cc_per_match[iuniq] = np.mean((line_cc[ww])[lindex[ww] == unq_val])
-        unq = unq.astype(np.int)
+        unq = unq.astype(int)
         # Assign the ID of this line to be wavelength whose index appears the largest number of times
         detids[dd] = linelist[unq[np.argmax(cnts)]]
         # Assign the cross-correlation of this line to be the average of when it was matched to this most often occurring wavelength
@@ -756,12 +756,12 @@ def solve_triangles(detlines, linelist, dindex, lindex, patt_dict=None):
     """
     nlines = detlines.size
     if patt_dict is None:
-        patt_dict = dict(acceptable=False, nmatch=0, ibest=-1, bwv=0., mask=np.zeros(nlines, dtype=np.bool))
+        patt_dict = dict(acceptable=False, nmatch=0, ibest=-1, bwv=0., mask=np.zeros(nlines, dtype=bool))
 
     # Find the best ID of each line
     detids = np.zeros(nlines)
     scores = ['None' for xx in range(nlines)]
-    mask = np.zeros(nlines, dtype=np.bool)
+    mask = np.zeros(nlines, dtype=bool)
     ngd_match = 0
     for dd in range(nlines):
         # Grab all the instances of this detected line's pixel position index
@@ -770,7 +770,7 @@ def solve_triangles(detlines, linelist, dindex, lindex, patt_dict=None):
             continue
         # Find the unique set of wavelength indices that this detected line has been matched to, and the number of times
         unq, cnts = np.unique(lindex[ww], return_counts=True)
-        unq = unq.astype(np.int)
+        unq = unq.astype(int)
         # Assign the ID of this line to be wavelength whose index appears the largest number of times
         detids[dd] = linelist[unq[np.argmax(cnts)]]
         # Give this ID a score based on the number of occurences
