@@ -4,6 +4,7 @@ from IPython import embed
 import numpy as np
 
 from pypeit.bitmask import BitMask
+from pypeit.images.bitmaskarray import BitMaskArray
 
 
 class ImageBitMask(BitMask):
@@ -12,6 +13,11 @@ class ImageBitMask(BitMask):
     image was masked.
     """
 
+    version = '1.1.0'
+    """
+    Bitmask key version.
+    """
+
     def __init__(self):
         # TODO:
         #   - Can IVAR0 and IVAR_NAN be consolidated into a single bit?
@@ -25,8 +31,14 @@ class ImageBitMask(BitMask):
                          IVAR0='Inverse variance is undefined',
                          IVAR_NAN='Inverse variance is NaN',
                          EXTRACT='Pixel masked during local skysub and extraction',
-                         BADSCALE='Bad image rescaling operation (e.g., flat value <= 0)',
-                         STCKMASK='All pixels masked in image stack')
+                         BADSCALE='Bad image rescaling operation (e.g., flat-field value <= 0)',
+                         STCKMASK='All pixels masked in image stack',
+                         USER='Pixel masked by user')
         super(ImageBitMask, self).__init__(list(mask_bits.keys()), descr=list(mask_bits.values()))
 
 
+class ImageBitMaskArray(BitMaskArray):
+    version = ImageBitMask.version
+    bitmask = ImageBitMask()
+
+
