@@ -8,6 +8,8 @@ import os
 import re
 from typing import List
 
+from IPython import embed
+
 import numpy as np
 
 from astropy import units
@@ -20,13 +22,11 @@ from pypeit import specobj
 from pypeit import io
 from pypeit.spectrographs.util import load_spectrograph
 from pypeit.core import parse
-from pypeit.images import detector_container
+from pypeit.images.detector_container import DetectorContainer
+from pypeit.images.mosaic import Mosaic
 from pypeit import slittrace
 from pypeit import utils
 
-from IPython import embed
-
-
 class SpecObjs:
     """
     Object to hold a set of :class:`~pypeit.specobj.SpecObj` objects
@@ -59,15 +59,17 @@ class SpecObjs:
         Also tag on the Header
 
         Args:
-            fits_file (str):
-            det (int, optional):
-                Only load SpecObj matching this det value
-            chk_version (:obj:`bool`):
+            fits_file (:obj:`str`):
+                The name of the fits file to read.
+            det (:obj:`str`, optional):
+                The string identifier for a detector or mosaic used to select
+                the loaded spectra.  If None, all spectra are loaded.
+            chk_version (:obj:`bool`, optional):
                 If False, allow a mismatch in datamodel to proceed
 
         Returns:
-            specobsj.SpecObjs
-
+            :class:`~pypeit.specobjs.SpecObjs`: The loaded spectra from the
+            provided fits file.
         """
         # HDUList
         hdul = io.fits_open(fits_file)
@@ -80,8 +82,20 @@ class SpecObjs:
         detector_hdus = {}
         # Loop for Detectors first as we need to add these to the objects
         for hdu in hdul[1:]:
-            if 'DETECTOR' in hdu.name:
-                detector_hdus[hdu.header['DET']] = detector_container.DetectorContainer.from_hdu(hdu)
+            if 'DETECTOR' not in hdu.name:
+                continue
+            if 'DMODCLS' not in hdu.header:
+                msgs.error('HDUs with DETECTOR in the name must have DMODCLS in their header.')
+            try:
+                dmodcls = eval(hdu.header['DMODCLS'])
+            except:
+                msgs.error(f"Unknown detector type datamodel class: {hdu.header['DMODCLS']}")
+            # NOTE: This requires that any "detector" datamodel class has a
+            # from_hdu method, and the name of the HDU must have a known format
+            # (e.g., 'DET01-DETECTOR').
+            _det = hdu.name.split('-')[0]
+            detector_hdus[_det] = dmodcls.from_hdu(hdu)
+
         # Now the objects
         for hdu in hdul[1:]:
             if 'DETECTOR' in hdu.name:
@@ -110,7 +124,7 @@ class SpecObjs:
                 specobjs = np.array(specobjs)
             self.specobjs = specobjs
 
-        self.header = header if header is not None else None
+        self.header = header
         self.hdul = None
 
         # Turn off attributes from here
@@ -197,7 +211,7 @@ class SpecObjs:
         flux = np.zeros((nspec, norddet))
         flux_ivar = np.zeros((nspec, norddet))
         flux_gpm = np.zeros((nspec, norddet), dtype=bool)
-        detector = np.zeros(norddet, dtype=int)
+        detector = [None]*norddet
         ech_orders = np.zeros(norddet, dtype=int)
 
         # TODO make the extraction that is desired OPT vs BOX an optional input variable.
@@ -218,7 +232,7 @@ class SpecObjs:
         # TODO JFH: Make this an atribute of the specobj by default.
         meta_spec['PYP_SPEC'] = self.header['PYP_SPEC']
         meta_spec['PYPELINE'] = self[0].PYPELINE
-        meta_spec['DET'] = detector
+        meta_spec['DET'] = np.array(detector)
         meta_spec['DISPNAME'] = self.header['DISPNAME']
         # Return
         if self[0].PYPELINE in ['MultiSlit', 'IFU'] and self.nobj == 1:
@@ -237,8 +251,8 @@ class SpecObjs:
 
         Args:
             multi_spec_det (list):
-                If there are multiple detectors arranged in the spectral direction, return the sobjs for
-                the standard on each detector.
+                If there are multiple detectors arranged in the spectral
+                direction, return the sobjs for the standard on each detector.
 
         Returns:
             SpecObj or SpecObjs or None
@@ -249,17 +263,29 @@ class SpecObjs:
         if 'MultiSlit' in pypeline or 'IFU' in pypeline:
             # Have to do a loop to extract the counts for all objects
             if self.OPT_COUNTS[0] is not None:
-                SNR = np.median(self.OPT_COUNTS*np.sqrt(self.OPT_COUNTS_IVAR), axis=1)
+                SNR = np.median(self.OPT_COUNTS * np.sqrt(self.OPT_COUNTS_IVAR), axis=1)
             elif self.BOX_COUNTS[0] is not None:
-                SNR = np.median(self.BOX_COUNTS*np.sqrt(self.BOX_COUNTS_IVAR), axis=1)
+                SNR = np.median(self.BOX_COUNTS * np.sqrt(self.BOX_COUNTS_IVAR), axis=1)
             else:
                 return None
+
             # For multiple detectors grab the requested detectors
             if multi_spec_det is not None:
+                # TODO: This is a hack assuming the integers in multi_spec_det
+                # are for *detectors*, not mosaics.
+                if any([isinstance(d, int) for d in multi_spec_det]):
+                    _multi_spec_det = [DetectorContainer.get_name(d) if isinstance(d, int) else d
+                                            for d in multi_spec_det]
+                else:
+                    _multi_spec_det = multi_spec_det
                 sobjs_std = SpecObjs(header=self.header)
                 # Now append the maximum S/N object on each detector
-                for idet in multi_spec_det:
+                for idet in _multi_spec_det:
                     this_det = self.DET == idet
+                    if not np.any(this_det):
+                        unique_det = np.unique(self.DET)
+                        msgs.error(f'No matches for {idet} in spec1d file.  Unique options found'
+                                   f"are {', '.join(unique_det)}.  Check usage of multi_spec_det.")
                     istd = SNR[this_det].argmax()
                     sobjs_std.add_sobj(self[this_det][istd])
             else: # For normal multislit take the brightest object
@@ -378,7 +404,7 @@ class SpecObjs:
             int:
         """
         if self[0].PYPELINE == 'Echelle':
-            indx = self.ECH_ORDERINDX == slitorder
+            indx = self.ECH_ORDER == slitorder
         elif self[0].PYPELINE == 'MultiSlit':
             indx = self.SLITID == slitorder
         elif self[0].PYPELINE == 'IFU':
@@ -409,14 +435,30 @@ class SpecObjs:
             msgs.error("The '{0:s}' PYPELINE is not defined".format(self[0].PYPELINE))
         return indx
 
-    def slitorder_objid_indices(self, slitorder, objid):
+    def slitorder_objid_indices(self, slitorder, objid, toler=5):
         """
-        Return the set of indices matching the input slit/order and the input objid
+        Return the set of indices matching the input slit/order and the input
+        objid
+        
+        Args:
+            slitorder (int):
+                Order/Spatial pixel value for slit of interest.
+            objid (int):
+                ID value for object of interest.
+            toler (int, optional):
+                Tolerance for slit spatial pixel values used for slit
+                identification. Default = 5
+
+        Returns:
+            :obj:`int`: Index value for input slit/order and object ID values
+            for specobjs object.
+
         """
+
         if self[0].PYPELINE == 'Echelle':
             indx = (self.ECH_ORDER == slitorder) & (self.ECH_OBJID == objid)
         elif self[0].PYPELINE == 'MultiSlit':
-            indx = (self.SLITID == slitorder) & (self.OBJID == objid)
+            indx = (np.abs(self.SLITID - slitorder) <= toler) & (self.OBJID == objid)
         elif self[0].PYPELINE == 'IFU':
             indx = (self.SLITID == slitorder) & (self.OBJID == objid)
         else:
@@ -433,19 +475,27 @@ class SpecObjs:
 
     def add_sobj(self, sobj):
         """
-        Add one or more SpecObj
+        Append one or more SpecObj to the existing set.
 
         Args:
-            sobj (SpecObj or list or ndarray):  One or more SpecObj objects
-
+            sobj (:class:`~pypeit.specobj.SpecObj`, :class:`~pypeit.specobjs.SpecObjs`, :obj:`list`, `numpy.ndarray`_):
+                One or more SpecObj objects to append.  If a list or array, the
+                elements of these must be instances of
+                :class:`~pypeit.specobj.SpecObj`.
         """
         if isinstance(sobj, specobj.SpecObj):
             self.specobjs = np.append(self.specobjs, [sobj])
-        elif isinstance(sobj, (np.ndarray,list)):
-            self.specobjs = np.append(self.specobjs, sobj)
-        elif isinstance(sobj, SpecObjs):
+            return
+        if isinstance(sobj, SpecObjs):
+            # TODO: Possible to instead do np.append(self.specobjs, sobj.specobjs)?
             for isobj in sobj:
                 self.specobjs = np.append(self.specobjs, isobj)
+            return
+        if not isinstance(sobj, (np.ndarray, list)):
+            msgs.error(f'Unable to add {type(sobj)} objects to SpecObjs')
+        if any([not isinstance(s, specobj.SpecObj) for s in sobj]):
+            msgs.error('List or arrays of objects to add must all be of type SpecObj.')
+        self.specobjs = np.append(self.specobjs, sobj)
 
     def remove_sobj(self, index):
         """
@@ -523,7 +573,7 @@ class SpecObjs:
             # For all, a new table is constructed with slice of all columns
             return SpecObjs(specobjs=self.specobjs[item], header=self.header)
 
-    def __getattr__(self, k):
+    def __getattr__(self, attr):
         """
         Overloaded to generate an array of attribute 'k' from the
         :class:`pypeit.specobj.SpecObj` objects.
@@ -531,13 +581,15 @@ class SpecObjs:
         First attempts to grab data from the Summary table, then the list
         """
         if len(self.specobjs) == 0:
-            raise ValueError("Empty specobjs")
+            raise ValueError('SpecObjs is empty!')
+        if attr in self.__dict__:  # any normal attributes are handled normally
+            return self.__dict__[attr]
         try:
-            lst = [getattr(specobj, k) for specobj in self.specobjs]
-        except ValueError:
-            raise ValueError("Attribute does not exist")
-        # Recast as an array
-        return lst_to_array(lst)
+            getattr(self.specobjs[0], attr)
+        except NameError:
+            raise NameError(f'{attr} is not an attribute of SpecObjs or SpecObj.')
+
+        return lst_to_array([getattr(specobj, attr) for specobj in self.specobjs])
 
     # Printing
     def __repr__(self):
@@ -554,6 +606,14 @@ class SpecObjs:
     def __len__(self):
         return len(self.specobjs)
 
+    @property
+    def size(self):
+        return self.specobjs.size
+
+    @property
+    def shape(self):
+        return self.specobjs.shape
+
     def write_to_fits(self, subheader, outfile, overwrite=True, update_det=None,
                       slitspatnum=None, history=None, debug=False):
         """
@@ -564,14 +624,16 @@ class SpecObjs:
             outfile (str):
             overwrite (bool, optional):
             slitspatnum (:obj:`str` or :obj:`list`, optional):
-                Restricted set of slits for reduction
+              Restricted set of slits for reduction.
+              If provided, do not clobber the existing file but only update
+              the indicated slits.  Useful for re-running on a subset of slits
             update_det (int or list, optional):
               If provided, do not clobber the existing file but only update
               the indicated detectors.  Useful for re-running on a subset of detectors
 
         """
-        if os.path.isfile(outfile) and (not overwrite):
-            msgs.warn("Outfile exists.  Set overwrite=True to clobber it")
+        if os.path.isfile(outfile) and not overwrite:
+            msgs.warn(f'{outfile} exits. Set overwrite=True to overwrite it.')
             return
 
         # If the file exists and update_det (and slit_spat_num) is provided, use the existing header
@@ -580,17 +642,18 @@ class SpecObjs:
         if os.path.isfile(outfile) and (update_det is not None or slitspatnum is not None):
             _specobjs = SpecObjs.from_fitsfile(outfile)
             mask = np.ones(_specobjs.nobj, dtype=bool)
-            # Update_det
-            if update_det is not None:
+            # Update
+            if slitspatnum is not None: # slitspatnum
+                dets, spat_ids = parse.parse_slitspatnum(slitspatnum)
+                for det, spat_id in zip(dets, spat_ids):
+                    mask[(_specobjs.DET == det) & (_specobjs.SLITID == spat_id)] = False
+            elif update_det is not None:
                 # Pop out those with this detector (and slit if slit_spat_num is provided)
                 for det in np.atleast_1d(update_det):
                     mask[_specobjs.DET == det] = False
-            elif slitspatnum is not None: # slitspatnum
-                dets, spat_ids = slittrace.parse_slitspatnum(slitspatnum)
-                for det, spat_id in zip(dets, spat_ids):
-                    mask[(_specobjs.DET == det) & (_specobjs.SLITID == spat_id)] = False
             _specobjs = _specobjs[mask]
             # Add in the new
+            # TODO: Is the loop necessary? add_sobj can take many SpecObj objects.
             for sobj in self.specobjs:
                 _specobjs.add_sobj(sobj)
         else:
@@ -627,20 +690,22 @@ class SpecObjs:
                 continue
             # HDUs
             if debug:
-                import pdb; pdb.set_trace()
+                raise NotImplementedError('Debugging for developers only.')
+                #embed()
+                #exit()
             shdul = sobj.to_hdu()
-            if len(shdul) == 2:  # Detector?
+            if len(shdul) not in [1, 2]:
+                msgs.error('CODING ERROR: SpecObj datamodel changed.  to_hdu should return 1 or 2 '
+                           'HDUs.  If returned, the 2nd one should be the detector/mosaic.')
+            if len(shdul) == 2:
                 detector_hdus[sobj['DET']] = shdul[1]
                 shdu = [shdul[0]]
-            elif len(shdul) == 1:  # Detector?
-                shdu = shdul
             else:
-                msgs.error("Should not get here...")
-            # Check -- If sobj had only 1 array, the BinTableHDU test will fail
-            assert len(shdu) == 1, 'Bad data model!!'
-            assert isinstance(shdu[0], fits.hdu.table.BinTableHDU), 'Bad data model2'
-            #shdu[0].header['DMODCLS'] = (self.__class__.__name__, 'Datamodel class')
-            #shdu[0].header['DMODVER'] = (self.version, 'Datamodel version')
+                shdu = shdul
+
+            if len(shdu) != 1 or not isinstance(shdu[0], fits.hdu.table.BinTableHDU):
+                msgs.error('CODING ERROR: SpecObj datamodel changed.')
+
             # Name
             shdu[0].name = sobj.NAME
             # Extension
@@ -653,11 +718,10 @@ class SpecObjs:
 
         # Deal with Detectors
         for key, item in detector_hdus.items():
-            # TODO - Add EXT to the primary header for these??
-            prefix = specobj.det_hdu_prefix(key)
+            prefix = item.header['name']
             # Name
             if prefix not in item.name:  # In case we are re-loading
-                item.name = specobj.det_hdu_prefix(key)+item.name
+                item.name = f'{prefix}-{item.name}'
             # Append
             hdus += [item]
 
@@ -670,7 +734,9 @@ class SpecObjs:
         # Finish
         hdulist = fits.HDUList(hdus)
         if debug:
-            embed()
+            raise NotImplementedError('Debugging for developers only.')
+             #embed()
+             #exit()
         hdulist.writeto(outfile, overwrite=overwrite)
         msgs.info("Wrote 1D spectra to {:s}".format(outfile))
         return
@@ -704,17 +770,16 @@ class SpecObjs:
                 spat_fracpos.append(specobj.SPAT_FRACPOS)
                 slits.append(specobj.SLITID)
                 names.append(specobj.NAME)
-                wave_rms.append(specobj.WAVE_RMS)
             elif pypeline == 'IFU':
                 spat_fracpos.append(specobj.SPAT_FRACPOS)
                 slits.append(specobj.SLITID)
                 names.append(specobj.NAME)
-                wave_rms.append(specobj.WAVE_RMS)
             elif pypeline == 'Echelle':
                 spat_fracpos.append(specobj.ECH_FRACPOS)
                 slits.append(specobj.ECH_ORDER)
                 names.append(specobj.ECH_NAME)
-                wave_rms.append(specobj.WAVE_RMS)
+            # Wave RMS
+            wave_rms.append(specobj.WAVE_RMS)
             # Boxcar width
             if specobj.BOX_RADIUS is not None:
                 slit_pix = 2.0 * specobj.BOX_RADIUS
@@ -748,15 +813,11 @@ class SpecObjs:
             # Manual extraction?
             manual_extract.append(specobj.hand_extract_flag)
             # Slitmask info
-            if specobj.MASKDEF_ID is not None:
-                maskdef_id.append(specobj.MASKDEF_ID)
-            if specobj.MASKDEF_OBJNAME is not None:
-                objname.append(specobj.MASKDEF_OBJNAME)
-            if specobj.RA is not None:
-                objra.append(specobj.RA)
-                objdec.append(specobj.DEC)
-            if specobj.MASKDEF_EXTRACT is not None:
-                maskdef_extract.append(specobj.MASKDEF_EXTRACT)
+            maskdef_id.append(specobj.MASKDEF_ID)
+            objname.append(specobj.MASKDEF_OBJNAME)
+            objra.append(specobj.RA)
+            objdec.append(specobj.DEC)
+            maskdef_extract.append(specobj.MASKDEF_EXTRACT)
 
         # Generate the table, if we have at least one source
         if len(names) > 0:
@@ -771,16 +832,17 @@ class SpecObjs:
                 obj_tbl['order'] = slits
                 obj_tbl['order'].format = 'd'
             obj_tbl['name'] = names
-            if len(maskdef_id) > 0:
+            if not np.all(np.array(maskdef_id) == None):
                 obj_tbl['maskdef_id'] = maskdef_id
-                obj_tbl['maskdef_id'].format = 'd'
-            if len(objname) > 0:
+            if not np.all(np.array(objname) == None):
                 obj_tbl['objname'] = objname
-            if len(objra) > 0:
+            if not np.all(np.array(objra) == None):
                 obj_tbl['objra'] = objra
-                obj_tbl['objra'].format = '.5f'
+                if None not in objra:
+                    obj_tbl['objra'].format = '.5f'
                 obj_tbl['objdec'] = objdec
-                obj_tbl['objdec'].format = '.5f'
+                if None not in objdec:
+                    obj_tbl['objdec'].format = '.5f'
             obj_tbl['spat_pixpos'] = spat_pixpos
             obj_tbl['spat_pixpos'].format = '.1f'
             obj_tbl['spat_fracpos'] = spat_fracpos
@@ -794,15 +856,17 @@ class SpecObjs:
             obj_tbl['s2n'] = s2n
             obj_tbl['s2n'].format = '.2f'
             # is this a forced extraction at the expected position from slitmask design?
-            if len(maskdef_extract) > 0:
-                obj_tbl['maskdef_extract'] = maskdef_extract
-            # only if manual extractions exist, print this
-            if np.any(manual_extract):
+            if not np.all(np.array(maskdef_extract) == None):
+                    obj_tbl['maskdef_extract'] = maskdef_extract
+            # only if any manual extraction exists, print this
+            if not np.all(np.array(manual_extract) == False):
                 obj_tbl['manual_extract'] = manual_extract
 
             # Wavelengths
-            obj_tbl['wv_rms'] = wave_rms
-            obj_tbl['wv_rms'].format = '.3f'
+            if not np.all(np.array(wave_rms) == None):
+                obj_tbl['wv_rms'] = wave_rms
+                if None not in wave_rms:
+                    obj_tbl['wv_rms'].format = '.3f'
             # Write
             obj_tbl.write(outfile,format='ascii.fixed_width', overwrite=True)
 
@@ -850,6 +914,48 @@ class SpecObjs:
 
         return groups
 
+#TODO Should this be a classmethod on specobjs??
+def get_std_trace(detname, std_outfile, chk_version=True):
+    """
+     Returns the trace of the standard.
+
+     Args:
+         det (:obj:`int`, :obj:`tuple`):
+             1-indexed detector(s) to process.
+         std_outfile (:obj:`str`):
+             Filename with the standard star spec1d file.  Can be None.
+     Returns:
+         `numpy.ndarray`_: Trace of the standard star on input detector.
+         Will be None if ``std_outfile`` is None, or if the selected detector/mosaic is not available
+         in the provided spec1d file.
+     """
+
+    sobjs = SpecObjs.from_fitsfile(std_outfile, chk_version=chk_version)
+    pypeline = sobjs.PYPELINE
+    # Does the detector match?
+    # TODO: Instrument specific logic here could be implemented with the
+    # parset. For example LRIS-B or LRIS-R we we would use the standard
+    # from another detector.
+
+    this_det = sobjs.DET == detname
+    if np.any(this_det):
+        sobjs_det = sobjs[this_det]
+        sobjs_std = sobjs_det.get_std()
+        # No standard extracted on this detector??
+        if sobjs_std is None:
+            return None
+        std_trace = sobjs_std.TRACE_SPAT
+        # flatten the array if this multislit
+        if 'MultiSlit' in pypeline:
+            std_trace = std_trace.flatten()
+        elif 'Echelle' in pypeline:
+            std_trace = std_trace.T
+        else:
+            msgs.error('Unrecognized pypeline')
+    else:
+        std_trace = None
+
+    return std_trace
 
 def lst_to_array(lst, mask=None):
     """
