@@ -3,10 +3,7 @@ Module for VLT X-Shooter
 
 .. include:: ../include/links.rst
 """
-import glob
-from pkg_resources import resource_filename
-
-from IPython import embed
+import os
 
 import numpy as np
 
@@ -20,7 +17,9 @@ from pypeit.core import parse
 from pypeit.core import framematch
 from pypeit.spectrographs import spectrograph
 from pypeit.images import detector_container
+from pypeit import data
 
+from IPython import embed
 
 class VLTXShooterSpectrograph(spectrograph.Spectrograph):
     """
@@ -29,6 +28,9 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
     ndet = 1
     telescope = telescopes.VLTTelescopePar()
     pypeline = 'Echelle'
+    url = 'https://www.eso.org/sci/facilities/paranal/instruments/xshooter.html'
+    ech_fixed_format = True
+    header_name = 'XSHOOTER'
 
     def init_meta(self):
         """
@@ -39,7 +41,8 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
         """
         self.meta = {}
         # Required (core)
-        self.meta['ra'] = dict(ext=0, card='RA', required_ftypes=['science', 'standard'])  # Need to convert to : separated
+        self.meta['ra'] = dict(ext=0, card='RA', 
+            required_ftypes=['science', 'standard'])  # Need to convert to : separated
         self.meta['dec'] = dict(ext=0, card='DEC', required_ftypes=['science', 'standard'])
         self.meta['target'] = dict(ext=0, card='OBJECT')
         self.meta['binning'] = dict(card=None, compound=True)
@@ -51,9 +54,10 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
         self.meta['dispname'] = dict(ext=0, card=None, default='default')
         self.meta['idname'] = dict(ext=0, card='HIERARCH ESO DPR CATG')
         self.meta['arm'] = dict(ext=0, card='HIERARCH ESO SEQ ARM')
-        # Dithering
+        self.meta['instrument'] = dict(ext=0, card='INSTRUME')
+        # Dithering -- Not required for redux
         self.meta['dither'] = dict(ext=0, card='HIERARCH ESO SEQ CUMOFF Y',
-                                   required_ftypes=['science', 'standard'])
+            required=False)  # This header card is *not* always present in science/standard frames
 
     def compound_meta(self, headarr, meta_key):
         """
@@ -140,7 +144,7 @@ class VLTXShooterSpectrograph(spectrograph.Spectrograph):
             return good_exp & (fitstbl['target'] == 'BIAS')
         if ftype == 'dark':
             return good_exp & (fitstbl['target'] == 'DARK')
-        if ftype in ['pixelflat', 'trace']:
+        if ftype in ['pixelflat', 'trace', 'illumflat']:
             # Flats and trace frames are typed together
             return good_exp & ((fitstbl['target'] == 'LAMP,DFLAT')
                                | (fitstbl['target'] == 'LAMP,QFLAT')
@@ -165,15 +169,16 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
     supported = True
     comment = 'See :doc:`xshooter`'
 
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
@@ -217,12 +222,15 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         par = super().default_pypeit_par()
 
         # Turn off illumflat
-        turn_off = dict(use_illumflat=False, use_biasimage=False, use_overscan=False, use_darkimage=False)
+        turn_off = dict(use_illumflat=False, use_biasimage=False, use_overscan=False,
+                        use_darkimage=False)
         par.reset_all_processimages_par(**turn_off)
-        # Require dark images to be subtracted from the flat images used for tracing, pixelflats, and illumflats
-        par['calibrations']['traceframe']['process']['use_darkimage'] = True
-        par['calibrations']['pixelflatframe']['process']['use_darkimage'] = True
-        par['calibrations']['illumflatframe']['process']['use_darkimage'] = True
+        # Require dark images to be subtracted from the flat images used for
+        # tracing, pixelflats, and illumflats
+        # par['calibrations']['traceframe']['process']['use_darkimage'] = True
+        # par['calibrations']['pixelflatframe']['process']['use_darkimage'] = True
+        # par['calibrations']['illumflatframe']['process']['use_darkimage'] = True
+        # TODO: `mask_cr` now defaults to True for darks.  Should this be turned off?
 
         # Is this needed below?
         par['scienceframe']['process']['sigclip'] = 20.0
@@ -258,7 +266,7 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         par['calibrations']['wavelengths']['reid_arxiv'] = 'vlt_xshooter_nir.fits'
         par['calibrations']['wavelengths']['cc_thresh'] = 0.50
         par['calibrations']['wavelengths']['cc_local_thresh'] = 0.50
-        par['calibrations']['wavelengths']['ech_fix_format'] = True
+#        par['calibrations']['wavelengths']['ech_fix_format'] = True
         # Echelle parameters
         par['calibrations']['wavelengths']['echelle'] = True
         par['calibrations']['wavelengths']['ech_nspec_coeff'] = 5
@@ -270,13 +278,17 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         par['calibrations']['flatfield']['tweak_slits_thresh'] = 0.90
         par['calibrations']['flatfield']['tweak_slits_maxfrac'] = 0.10
 
+        # Standards
+        par['calibrations']['standardframe']['process']['mask_cr'] = False
+
         # Extraction
         par['reduce']['skysub']['bspline_spacing'] = 0.8
         par['reduce']['skysub']['global_sky_std']  = False # Do not perform global sky subtraction for standard stars
         par['reduce']['extraction']['model_full_slit'] = True  # local sky subtraction operates on entire slit
         par['reduce']['findobj']['trace_npoly'] = 8
-        par['reduce']['findobj']['find_npoly_cont'] = 0  # Continnum order for determining thresholds
-        par['reduce']['findobj']['find_cont_fit'] = False  # Don't attempt to fit a continuum to the trace rectified image
+        par['reduce']['findobj']['maxnumber_sci'] = 2  # Assume that there is only one object on the slit.
+        par['reduce']['findobj']['maxnumber_std'] = 1  # Assume that there is only one object on the slit.
+
 
         # The settings below enable X-shooter dark subtraction from the traceframe and pixelflatframe, but enforce
         # that this bias won't be subtracted from other images. It is a hack for now, because eventually we want to
@@ -293,29 +305,10 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         # Sensitivity function parameters
         par['sensfunc']['algorithm'] = 'IR'
         par['sensfunc']['polyorder'] = 8
-        par['sensfunc']['IR']['telgridfile'] \
-                = resource_filename('pypeit',
-                                    '/data/telluric/TelFit_Paranal_NIR_9800_25000_R25000.fits')
+        par['sensfunc']['IR']['telgridfile'] = 'TelFit_Paranal_NIR_9800_25000_R25000.fits'
 
         return par
 
-# TODO: out-of-date
-#    def check_headers(self, headers):
-#        """
-#        Check headers match expectations for an VLT/XSHOOTER exposure.
-#
-#        See also
-#        :func:`pypeit.spectrographs.spectrograph.Spectrograph.check_headers`.
-#
-#        Args:
-#            headers (list):
-#                A list of headers read from a fits file
-#        """
-#        expected_values = { '0.INSTRUME': 'XSHOOTER',
-#                            '0.HIERARCH ESO SEQ ARM': 'NIR',
-#                            '0.NAXIS': 2 }
-#        super(VLTXShooterNIRSpectrograph, self).check_headers(headers,
-#                                                              expected_values=expected_values)
 
     def init_meta(self):
         """
@@ -330,6 +323,9 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
 
         # Required
         self.meta['decker'] = dict(ext=0, card='HIERARCH ESO INS OPTI5 NAME')
+    
+        # Dark-flat identification via exposure number
+        self.meta['seq_expno'] = dict(ext=0, card='HIERARCH ESO TPL EXPNO')
 
     def pypeit_file_keys(self):
         """
@@ -366,6 +362,12 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
             exposures in ``fitstbl`` that are ``ftype`` type frames.
         """
         good_exp = framematch.check_frame_exptime(fitstbl['exptime'], exprng)
+
+        # Default NIR calibration behavior is to take flat/darks in sequence
+        #  These are marked by the seq_expno column
+        good_flat_seq = np.array([seq is not None and int(seq) % 2 == 1 for seq in fitstbl['seq_expno']])
+        good_dark_seq = np.array([seq is not None and int(seq) % 2 == 0 for seq in fitstbl['seq_expno']])
+
         # TODO: Allow for 'sky' frame type, for now include sky in
         # 'science' category
         if ftype == 'science':
@@ -376,13 +378,24 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
             return good_exp & (fitstbl['target'] == 'STD,FLUX')
         if ftype == 'bias':
             return good_exp & (fitstbl['target'] == 'BIAS')
-        if ftype == 'dark':
+        if ftype == 'sky':
             return good_exp & (fitstbl['target'] == 'DARK')
+        
         if ftype in ['pixelflat', 'trace']:
             # Flats and trace frames are typed together
-            return good_exp & ((fitstbl['target'] == 'LAMP,DFLAT')
+            # Lamp on flats are taken first (odd exposure number)
+            return good_exp & (((fitstbl['target'] == 'LAMP,DFLAT')
                                | (fitstbl['target'] == 'LAMP,QFLAT')
                                | (fitstbl['target'] == 'LAMP,FLAT'))
+                               & good_flat_seq)
+        
+        if ftype in ['dark']:
+            # Lamp off flats are taken second (even exposure number)
+            return good_exp & (((fitstbl['target'] == 'LAMP,DFLAT')
+                                | (fitstbl['target'] == 'LAMP,QFLAT')
+                                | (fitstbl['target'] == 'LAMP,FLAT'))
+                               & good_dark_seq)
+        
         if ftype == 'pinhole':
             # Don't type pinhole
             return np.zeros(len(fitstbl), dtype=bool)
@@ -423,12 +436,13 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
         bpm_img = super().bpm(filename, det, shape=shape, msbias=msbias)
 
         if det == 1:
-            bpm_dir = resource_filename('pypeit', 'data/static_calibs/vlt_xshoooter/')
+            bpm_dir = os.path.join(data.Paths.static_calibs, 'vlt_xshoooter')
             try :
-                bpm_loc = np.loadtxt(bpm_dir+'BP_MAP_RP_NIR.dat',usecols=(0,1))
+                bpm_loc = np.loadtxt(os.path.join(bpm_dir, 'BP_MAP_RP_NIR.dat'),
+                                     usecols=(0,1))
             except IOError :
                 msgs.warn('BP_MAP_RP_NIR.dat not present in the static database')
-                bpm_fits = io.fits_open(bpm_dir+'BP_MAP_RP_NIR.fits.gz')
+                bpm_fits = io.fits_open(os.path.join(bpm_dir, 'BP_MAP_RP_NIR.fits.gz'))
                 # ToDo: this depends on datasec, biassec, specflip, and specaxis
                 #       and should become able to adapt to these parameters.
                 # Flipping and shifting BPM to match the PypeIt format
@@ -444,7 +458,8 @@ class VLTXShooterNIRSpectrograph(VLTXShooterSpectrograph):
                 filt_bpm = bpm_data_pypeit[1:y_len,1:x_len]>100.
                 y_bpm, x_bpm = np.where(filt_bpm)
                 bpm_loc = np.array([y_bpm,x_bpm]).T
-                np.savetxt(bpm_dir+'BP_MAP_RP_NIR.dat', bpm_loc, fmt=['%d','%d'])
+                np.savetxt(os.path.join(bpm_dir, 'BP_MAP_RP_NIR.dat'), bpm_loc,
+                           fmt=['%d','%d'])
             finally :
                 bpm_img[bpm_loc[:,0].astype(int),bpm_loc[:,1].astype(int)] = 1.
 
@@ -539,22 +554,24 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
     supported = True
     comment = 'See :doc:`xshooter`'
 
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
         # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        # TODO: Could this be detector dependent??
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict = dict(
@@ -590,7 +607,8 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
 
         # Adjustments to parameters for VIS
         turn_on = dict(use_biasimage=False, use_overscan=True, overscan_method='median',
-                       use_darkimage=False, use_illumflat=False, use_pixelflat=False)
+                       use_darkimage=False, use_illumflat=False, use_pixelflat=False,
+                       use_specillum=False)
         par.reset_all_processimages_par(**turn_on)
 
         # X-SHOOTER arcs/tilts are also have different binning with bias
@@ -598,16 +616,15 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         # calibrations since it appears to be a different amplifier readout
         par['calibrations']['traceframe']['process']['overscan_method'] = 'median'
 
+        par['scienceframe']['process']['use_biasimage']=True
+        par['scienceframe']['process']['use_illumflat']=True
+        par['scienceframe']['process']['use_pixelflat']=True
         # Right now we are using the overscan and not biases becuase the
         # standards are read with a different read mode and we don't yet have
         # the option to use different sets of biases for different standards,
         # or use the overscan for standards but not for science frames
-        par['scienceframe']['process']['use_biasimage']=True
-        par['scienceframe']['process']['use_illumflat']=True
-        par['scienceframe']['process']['use_pixelflat']=True
         par['calibrations']['standardframe']['process']['use_illumflat']=True
         par['calibrations']['standardframe']['process']['use_pixelflat']=True
-        #par['scienceframe']['useframe'] ='overscan'
 
         par['calibrations']['slitedges']['edge_thresh'] = 8.0
         par['calibrations']['slitedges']['fit_order'] = 8
@@ -619,11 +636,11 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         # These are the defaults
         par['calibrations']['tilts']['tracethresh'] = 15
         par['calibrations']['tilts']['spat_order'] =  3
-        par['calibrations']['tilts']['spec_order'] =  5 # [5, 5, 5] + 12*[7] # + [5]
+        par['calibrations']['tilts']['spec_order'] =  5 
 
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['ThAr_XSHOOTER_VIS']
-        # This is for 1x1 binning. TODO GET BINNING SORTED OUT!!
+        # The following is for 1x1 binning. TODO GET BINNING SORTED OUT!!
         par['calibrations']['wavelengths']['rms_threshold'] = 0.50
         par['calibrations']['wavelengths']['sigdetect'] = 5.0
         par['calibrations']['wavelengths']['n_final'] = [3] + 13*[4] + [3]
@@ -636,7 +653,7 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         par['calibrations']['wavelengths']['reid_arxiv'] = 'vlt_xshooter_vis1x1.fits'
         par['calibrations']['wavelengths']['cc_thresh'] = 0.50
         par['calibrations']['wavelengths']['cc_local_thresh'] = 0.50
-        par['calibrations']['wavelengths']['ech_fix_format'] = True
+#        par['calibrations']['wavelengths']['ech_fix_format'] = True
         # Echelle parameters
         par['calibrations']['wavelengths']['echelle'] = True
         par['calibrations']['wavelengths']['ech_nspec_coeff'] = 4
@@ -654,17 +671,14 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
         par['reduce']['extraction']['model_full_slit'] = True
         # Mask 3 edges pixels since the slit is short, insted of default (5,5)
         par['reduce']['findobj']['find_trim_edge'] = [3,3]
+        par['reduce']['findobj']['maxnumber_sci'] = 2  # Assume that there is only one object on the slit.
+        par['reduce']['findobj']['maxnumber_std'] = 1  # Assume that there is only one object on the slit.
         # Continnum order for determining thresholds
-        par['reduce']['findobj']['find_npoly_cont'] = 0
-        # Don't attempt to fit a continuum to the trace rectified image
-        par['reduce']['findobj']['find_cont_fit'] = False
 
         # Sensitivity function parameters
         par['sensfunc']['algorithm'] = 'IR'
-        par['sensfunc']['polyorder'] = 11
-        par['sensfunc']['IR']['telgridfile'] \
-                = resource_filename('pypeit',
-                                    '/data/telluric/TelFit_Paranal_VIS_4900_11100_R25000.fits')
+        par['sensfunc']['polyorder'] = [9, 11, 11, 9, 9, 8, 8, 7, 7, 7, 7, 7, 7, 7, 7]
+        par['sensfunc']['IR']['telgridfile'] = 'TelFit_Paranal_VIS_4900_11100_R25000.fits'
         return par
 
     def init_meta(self):
@@ -680,6 +694,7 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
 
         # Required
         self.meta['decker'] = dict(ext=0, card='HIERARCH ESO INS OPTI4 NAME')
+    
 
     @property
     def norders(self):
@@ -808,39 +823,42 @@ class VLTXShooterVISSpectrograph(VLTXShooterSpectrograph):
 
         if det == 1:
             bpm_img[2912//binspectral_bpm:,842//binspatial_bpm:844//binspatial_bpm] = 1.
+            bpm_img[3548//binspectral_bpm:,1249//binspatial_bpm:1252//binspatial_bpm] = 1.
         return bpm_img
 
 
 class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
     """
-    Child to handle VLT/XSHOOTER specific code
+    Child to handle VLT/XSHOOTER specific code for the UVB arm
     """
 
     name = 'vlt_xshooter_uvb'
     camera = 'XShooter_UVB'
+    supported = True
     comment = 'See :doc:`xshooter`'
     
-    def get_detector_par(self, hdu, det):
+    def get_detector_par(self, det, hdu=None):
         """
         Return metadata for the selected detector.
 
         Args:
-            hdu (`astropy.io.fits.HDUList`_):
-                The open fits file with the raw image of interest.
             det (:obj:`int`):
                 1-indexed detector number.
+            hdu (`astropy.io.fits.HDUList`_, optional):
+                The open fits file with the raw image of interest.  If not
+                provided, frame-dependent parameters are set to a default.
 
         Returns:
             :class:`~pypeit.images.detector_container.DetectorContainer`:
             Object with the detector metadata.
         """
         # Binning
-        binning = self.get_meta_value(self.get_headarr(hdu), 'binning')  # Could this be detector dependent??
+        binning = '1,1' if hdu is None else self.get_meta_value(self.get_headarr(hdu), 'binning')
 
         # Detector 1
         detector_dict = dict(
-            binning         =binning,
-            det             =1,
+            binning         = binning,
+            det             = 1,
             dataext         = 0,
             specaxis        = 0,
             specflip        = True,
@@ -848,7 +866,7 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
             platescale      = 0.161, # average from order 14 and order 24, see manual
             darkcurr        = 0.0,
             saturation      = 65000.,
-            nonlinear       = 0.86,
+            nonlinear       = 0.86,  
             mincounts       = -1e10,
             numamplifiers   = 1,
             gain            = np.atleast_1d(1.61),
@@ -870,6 +888,16 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         """
         par = super().default_pypeit_par()
 
+        # Adjustments to parameters for UVB (following VIS)
+        turn_on = dict(use_biasimage=False, use_overscan=True, overscan_method='median',
+                       use_darkimage=False, use_illumflat=False, use_pixelflat=False,
+                       use_specillum=False)
+        par.reset_all_processimages_par(**turn_on)
+
+        # X-SHOOTER arcs/tilts are also have different binning with bias
+        # frames, so don't use bias frames. Don't use the biases for any
+        # calibrations since it appears to be a different amplifier readout
+
         # Adjustments to slit and tilts for UVB
         par['calibrations']['slitedges']['edge_thresh'] = 8.
         par['calibrations']['slitedges']['max_shift_adj'] = 0.5
@@ -877,32 +905,47 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         par['calibrations']['slitedges']['left_right_pca'] = True
         par['calibrations']['slitedges']['length_range'] = 0.3
 
-        par['calibrations']['arcframe']['process']['overscan_method'] = 'median'
-        par['calibrations']['traceframe']['process']['overscan_method'] = 'median'
-        # X-SHOOTER UVB arcs/tilts have different binning with bias frames
-        par['calibrations']['arcframe']['process']['use_biasimage'] = False
-        par['calibrations']['tiltframe']['process']['use_biasimage'] = False
-
         # 1D wavelength solution
         par['calibrations']['wavelengths']['lamps'] = ['ThAr_XSHOOTER_UVB']
-        # TODO: This is a KLUDGE; default_pypeit_par should be a
-        # staticmethod meaning it should not depend on self
-        #par['calibrations']['wavelengths']['nonlinear_counts'] = self.detector[0]['nonlinear'] * self.detector[0]['saturation']
-        par['calibrations']['wavelengths']['rms_threshold'] = 0.50 # This is for 1x1 binning. TODO GET BINNING SORTED OUT!!
-        par['calibrations']['wavelengths']['sigdetect'] = 5.0
+        par['calibrations']['wavelengths']['n_final'] = [3] + 10*[4] 
+        par['calibrations']['wavelengths']['rms_threshold'] = 0.60 
+        par['calibrations']['wavelengths']['sigdetect'] = 3.0 # Pretty faint lines in places
         # Reidentification parameters
         par['calibrations']['wavelengths']['method'] = 'reidentify'
-        # ToDo the arxived solution is for 1x1 binning. It needs to be generalized for different binning!
-        par['calibrations']['wavelengths']['reid_arxiv'] = 'vlt_xshooter_uvb1x1_iraf.json'
-        par['calibrations']['wavelengths']['ech_fix_format'] = True
+        par['calibrations']['wavelengths']['reid_arxiv'] = 'vlt_xshooter_uvb1x1.fits'
+#        par['calibrations']['wavelengths']['ech_fix_format'] = True
         # Echelle parameters
         par['calibrations']['wavelengths']['echelle'] = True
         par['calibrations']['wavelengths']['ech_nspec_coeff'] = 4
-        par['calibrations']['wavelengths']['ech_norder_coeff'] = 5
+        par['calibrations']['wavelengths']['ech_norder_coeff'] = 4
         par['calibrations']['wavelengths']['ech_sigrej'] = 3.0
+        
+        par['calibrations']['wavelengths']['cc_thresh'] = 0.50
+        par['calibrations']['wavelengths']['cc_local_thresh'] = 0.50
 
-        # TODO FIX THIS TO USE BIASES!!
-        par['scienceframe']['useframe'] ='overscan'
+        # Right now we are using the overscan and not biases becuase the
+        # standards are read with a different read mode and we don't yet have
+        # the option to use different sets of biases for different standards,
+        # or use the overscan for standards but not for science frames
+        par['scienceframe']['process']['use_biasimage']=True
+        par['scienceframe']['process']['use_illumflat']=True
+        par['scienceframe']['process']['use_pixelflat']=True
+        par['calibrations']['standardframe']['process']['use_illumflat']=True
+        par['calibrations']['standardframe']['process']['use_pixelflat']=True
+
+
+        # Extraction
+        par['reduce']['skysub']['bspline_spacing'] = 0.5
+        par['reduce']['skysub']['global_sky_std'] = False
+        par['reduce']['extraction']['model_full_slit'] = True
+        # Mask 3 edges pixels since the slit is short, insted of default (5,5)
+        par['reduce']['findobj']['find_trim_edge'] = [3,3]
+        # Continnum order for determining thresholds
+        #par['reduce']['findobj']['find_npoly_cont'] = 0
+        # Don't attempt to fit a continuum to the trace rectified image
+        #par['reduce']['findobj']['find_cont_fit'] = False
+        par['reduce']['findobj']['maxnumber_sci'] = 2  # Assume that there is a max of 2 objects on the slit
+        par['reduce']['findobj']['maxnumber_std'] = 1  # Assume that there is only one object on the slit.
 
         return par
 
@@ -915,12 +958,59 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         """
         super().init_meta()
         # Add the name of the dispersing element
-        # dispangle and filter1 are not defined for Shane Kast Blue
 
         # Required
         self.meta['decker'] = dict(ext=0, card='HIERARCH ESO INS OPTI3 NAME')
 
-    def order_platescale(self, binning = None):
+    @property
+    def norders(self):
+        """
+        Number of orders observed for this spectograph.
+        """
+        return 11
+
+    @property
+    def order_spat_pos(self):
+        """
+        Return the expected spatial position of each echelle order.
+
+        The following lines generated the values below:
+
+        .. code-block:: python
+
+            from pypeit import edgetrace
+            edges = edgetrace.EdgeTraceSet.from_file('MasterEdges_A_1_DET01.fits.gz')
+
+            nrm_edges = edges.edge_fit[edges.nspec//2,:] / edges.nspat
+            slit_cen = ((nrm_edges + np.roll(nrm_edges,1))/2)[np.arange(nrm_edges.size//2)*2+1]
+
+        """
+        # This starts by ignoring the first, partial order (25?)
+        #  Order 24 is very faint and not included here
+        #  Order 12 is very faint and not included here (the flat crashes out with issues..)
+        return np.array([0.32671887, 0.39553878, 0.45989826, 0.52009878, 0.5764598,
+            0.62917188, 0.67859507, 0.72482729, 0.76815531, 0.80879042,
+            0.84700373])#, 0.88317493])
+
+    @property
+    def orders(self):
+        """
+        Return the order number for each echelle order.
+        """
+        return np.arange(23, 12, -1, dtype=int)  # 11 orders; the reddest is too faint to use
+        #return np.arange(23, 11, -1, dtype=int)   # 12 orders
+
+    @property
+    def spec_min_max(self):
+        """
+        Return the minimum and maximum spectral pixel expected for the
+        spectral range of each order.
+        """
+        spec_max = np.asarray([4000]*13)# + [3000])
+        spec_min = np.asarray([0]*13)
+        return np.vstack((spec_min, spec_max))
+
+    def order_platescale(self, order_vec, binning = None):
         """
         Return the platescale for each echelle order.
 
@@ -937,15 +1027,15 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
             `numpy.ndarray`_: An array with the platescale for each order
             provided by ``order``.
         """
-        # TODO: So UVB is not supported?
-        msgs.error("REFACTOR")
-
         binspectral, binspatial = parse.parse_binning(binning)
 
         # ToDO Either assume a linear trend or measure this
         # X-shooter manual says, but gives no exact numbers per order.
         # UVB: 65.9 pixels (0.167“/pix) at order 14 to 70.8 pixels (0.155”/pix) at order 24
 
+        # Assume a simple linear trend
+        plate_scale = 0.155 + (order_vec - 24)*(0.155-0.167)/(24 - 14)
+
         # Right now I just took the average
         return np.full(self.norders, 0.161)*binspatial
 
@@ -979,10 +1069,7 @@ class VLTXShooterUVBSpectrograph(VLTXShooterSpectrograph):
         # Call the base-class method to generate the empty bpm
         bpm_img = super().bpm(filename, det, shape=shape, msbias=msbias)
 
-        if det == 1:
-            # TODO: This is for the 1x1 binning it should
-            # change for other binning
-            bpm_img[:2369,1326:1328] = 1.
+        # TODO -- Mask bad column if it is problematic (it isn't so far)
 
         return bpm_img
 
