@@ -8,7 +8,6 @@ General utility functions.
 import os
 import inspect
 import pickle
-import warnings
 import itertools
 from glob import glob
 from typing import List
@@ -17,9 +16,8 @@ from IPython import embed
 
 import numpy as np
 from numpy.lib.stride_tricks import as_strided
-import bottleneck
 
-from scipy import interpolate, ndimage
+from scipy import ndimage
 
 import matplotlib
 from matplotlib import pyplot as plt
@@ -27,8 +25,8 @@ from matplotlib import pyplot as plt
 from astropy import units
 from astropy import stats
 
-from pypeit.core import pydl
 from pypeit import msgs
+from pypeit.move_median import move_median
 
 
 def get_time_string(codetime):
@@ -352,6 +350,37 @@ def nearest_unmasked(arr, use_indices=False):
     return np.ma.argmin(nearest, axis=1)
 
 
+def contiguous_true(m):
+    """
+    Find contiguous regions of True values in a boolean numpy array.
+
+    This is identically what is done by `numpy.ma.flatnotmasked_contiguous`_,
+    except the argument is the mask, not a masked array, and it selects
+    contiguous True regions instead of contiguous False regions.
+
+    Args:
+        m (array-like):
+            A boolean array.  Must be 1D.
+
+    Returns:
+        :obj:`list`: A list of slice objects that select contiguous regions of
+        True values in the provided array.
+    """
+    _m = np.atleast_1d(m)
+    if _m.ndim > 1:
+        raise ValueError('contiguous_true only accepts 1D arrays.')
+    if not np.any(_m):
+        return [slice(0, _m.size)]
+    i = 0
+    result = []
+    for (k, g) in itertools.groupby(_m.ravel()):
+        n = len(list(g))
+        if k:
+            result.append(slice(i, i + n))
+        i += n
+    return result
+
+
 def boxcar_smooth_rows(img, nave, wgt=None, mode='nearest', replace='original'):
     """
     Boxcar smooth an image along their first axis (rows).
@@ -477,9 +506,9 @@ def rebin_evlist(frame, newshape):
     # https://scipy-cookbook.readthedocs.io/items/Rebinning.html
     shape = frame.shape
     lenShape = len(shape)
-    factor = np.asarray(shape)/np.asarray(newshape)
+    factor = (np.asarray(shape)/np.asarray(newshape)).astype(int)
     evList = ['frame.reshape('] + \
-             ['int(newshape[%d]),int(factor[%d]),'% (i, i) for i in range(lenShape)] + \
+             ['int(newshape[%d]),factor[%d],'% (i, i) for i in range(lenShape)] + \
              [')'] + ['.sum(%d)' % (i+1) for i in range(lenShape)] + \
              ['/factor[%d]' % i for i in range(lenShape)]
     return eval(''.join(evList))
@@ -628,7 +657,7 @@ def fast_running_median(seq, window_size):
     # pad the array for the reflection
     seq_pad = np.concatenate((seq[0:window_size][::-1],seq,seq[-1:(-1-window_size):-1]))
 
-    result = bottleneck.move_median(seq_pad, window=window_size)
+    result = move_median.move_median(seq_pad, window_size)
 
     # This takes care of the offset produced by the original code deducec by trial and error comparison with
     # scipy.ndimage.filters.medfilt
@@ -870,83 +899,6 @@ def polyval2d(x, y, m):
     return z
 
 
-'''
-def robust_polyfit(xarray, yarray, order, weights=None, maxone=True, sigma=3.0,
-                   function="polynomial", initialmask=None, forceimask=False,
-                   minx=None, maxx=None, guesses=None, bspline_par=None, verbose=True):
-    """
-    A robust (equally weighted) polynomial fit is performed to the xarray, yarray pairs
-    mask[i] = 1 are masked values
-
-    :param xarray: independent variable values
-    :param yarray: dependent variable values
-    :param order: the order of the polynomial to be used in the fitting
-    :param weights: weights to be used in the fitting (weights = 1/sigma)
-    :param maxone: If True, only the most deviant point in a given iteration will be removed
-    :param sigma: confidence interval for rejection
-    :param function: which function should be used in the fitting (valid inputs: 'polynomial', 'legendre', 'chebyshev', 'bspline')
-    :param initialmask: a mask can be supplied as input, these values will be masked for the first iteration. 1 = value masked
-    :param forceimask: if True, the initialmask will be forced for all iterations
-    :param minx: minimum value in the array (or the left limit for a legendre/chebyshev polynomial)
-    :param maxx: maximum value in the array (or the right limit for a legendre/chebyshev polynomial)
-    :return: mask, ct -- mask is an array of the masked values, ct is the coefficients of the robust polyfit.
-    """
-    # Setup the initial mask
-    if initialmask is None:
-        mask = np.zeros(xarray.size, dtype=np.int)
-        if forceimask:
-            msgs.warn("Initial mask cannot be enforced -- no initital mask supplied")
-            forceimask = False
-    else:
-        mask = initialmask.copy()
-    mskcnt = np.sum(mask)
-    # Iterate, and mask out new values on each iteration
-    ct = guesses
-
-    while True:
-        w = np.where(mask == 0)
-        xfit = xarray[w]
-        yfit = yarray[w]
-        if weights is not None:
-            wfit = weights[w]
-        else:
-            wfit = None
-        ct = func_fit(xfit, yfit, function, order, w=wfit,
-                      guesses=ct, minx=minx, maxx=maxx, bspline_par=bspline_par)
-        yrng = func_val(ct, xarray, function, minx=minx, maxx=maxx)
-        sigmed = 1.4826*np.median(np.abs(yfit-yrng[w]))
-        #if xarray.size-np.sum(mask) <= order+2: JFH fixed this bug
-        if xarray.size - np.sum(mask) <= order + 1:
-            if verbose:
-                msgs.warn("More parameters than data points - fit might be undesirable")
-            break  # More data was masked than allowed by order
-        if maxone:  # Only remove the most deviant point
-            tst = np.abs(yarray[w]-yrng[w])
-            m = np.argmax(tst)
-            if tst[m] > sigma*sigmed:
-                mask[w[0][m]] = 1
-        else:
-            if forceimask:
-                w = np.where((np.abs(yarray-yrng) > sigma*sigmed) | (initialmask==1))
-            else:
-                w = np.where(np.abs(yarray-yrng) > sigma*sigmed)
-            mask[w] = 1
-        if mskcnt == np.sum(mask): break  # No new values have been included in the mask
-        mskcnt = np.sum(mask)
-
-    # Final fit
-    w = np.where(mask == 0)
-    xfit = xarray[w]
-    yfit = yarray[w]
-    if weights is not None:
-        wfit = weights[w]
-    else:
-        wfit = None
-    ct = func_fit(xfit, yfit, function, order, w=wfit, minx=minx, maxx=maxx, bspline_par=bspline_par)
-    return mask, ct
-'''
-
-
 def subsample(frame):
     """
     Used by LACosmic
@@ -998,8 +950,7 @@ def yamlify(obj, debug=False):
 
     Recursively process an object so it can be serialised for yaml.
 
-    Based on jsonify in `linetools
-    <https://pypi.python.org/pypi/linetools>`_.
+    Based on jsonify in `linetools`_.
 
     Also found in desiutils
 
@@ -1021,8 +972,10 @@ def yamlify(obj, debug=False):
         :class:`numpy.ndarray` is converted to :class:`list`,
         :class:`numpy.int64` is converted to :class:`int`, etc.
     """
+    # TODO: Change to np.floating?
     if isinstance(obj, (np.float64, np.float32)):
         obj = float(obj)
+    # TODO: Change to np.integer?
     elif isinstance(obj, (np.int32, np.int64, np.int16)):
         obj = int(obj)
     elif isinstance(obj, np.bool_):
@@ -1030,7 +983,11 @@ def yamlify(obj, debug=False):
 #    elif isinstance(obj, bytes):
 #        obj = obj.decode('utf-8')
     elif isinstance(obj, (np.string_, str)):
-        obj = str(obj)
+        # Worry about colons!
+        if ':' in obj:
+            obj = '"'+str(obj)+'"'
+        else:
+            obj = str(obj)
     elif isinstance(obj, units.Quantity):
         try:
             obj = obj.value.tolist()
@@ -1401,3 +1358,69 @@ def DFS(v: int, visited: List[bool], group: List[int], adj: np.ndarray):
             neighbors = [i for i in range(len(adj[u])) if adj[u,i]]
             for neighbor in neighbors:
                 stack.append(neighbor)
+
+def list_of_spectral_lines():
+    """ Generate a list of spectral lines
+
+    Returns:
+        tuple: np.ndarray, np.ndarray
+    """
+    # spectral features
+    CIVnam1, CIVwav1='CIV', 1548.
+    CIVnam2, CIVwav2='CIV', 1550.
+
+    HeIInam0, HeIIwav0='HeII', 1640.
+
+    OIIInam01, OIIIwav01='OIII]', 1661.
+    OIIInam02, OIIIwav02='OIII]', 1666.
+
+    SiIIInam1, SiIIIwav1='SiIII]', 1882.
+    SiIIInam2, SiIIIwav2='SiIII]', 1892.
+
+    CIIInam1, CIIIwav1='CIII]', 1907.
+    CIIInam2, CIIIwav2='CIII]', 1909.
+
+    Lyalphanam, Lyalphawav='Lyalpha', 1215.7
+    OIInam1, OIIwav1='[OII]', 3726
+    OIInam2, OIIwav2='[OII]', 3729
+    OIIInam1, OIIIwav1='[OIII]', 5007.
+    OIIInam2, OIIIwav2='[OIII]', 4959.
+    OIIInam3, OIIIwav3='[OIII]', 4363.
+    Halphanam, Halphawav='Halpha', 6563.
+    Hbetanam, Hbetawav='Hbeta', 4861.
+    Hdeltanam, Hdeltawav='Hdelta', 4101.
+    Hgammanam, Hgammawav='Hgamma', 4341.
+
+    NeIIInam, NeIIIwav = '[NeIII]', 3869.
+    NeVnam, NeVwav = '[NeV]', 3426.
+    SIInam1, SIIwav1 = '[SII]', 6716.
+    SIInam2, SIIwav2 = '[SII]', 6716.
+
+
+    ##absorption
+    H13nam, H13wav = 'H13', 3734.
+    H12nam, H12wav = 'H12', 3750.
+    H11nam, H11wav = 'H11', 3771.
+    H10nam, H10wav = 'H10', 3798.
+    H9nam, H9wav = 'H9', 3835.
+    H8nam, H8wav = 'H8', 3889.
+    HeInam, HeIwav = 'HeI', 3889.
+
+    CAII_Knam, CaII_Kwav = 'CaK', 3934.
+    CAII_Hnam, CaII_Hwav = 'CaH', 3968.
+
+    Gbandnam, Gbandwav = 'Gband', 4305.
+
+    line_names=np.array([CIVnam1, CIVnam2, HeIInam0, OIIInam01, OIIInam02, SiIIInam1, SiIIInam2,
+                         CIIInam1, CIIInam2, Lyalphanam, OIInam1, OIInam2, OIIInam1, OIIInam2,
+                         OIIInam3, Halphanam, Hbetanam, Hdeltanam, Hgammanam, NeIIInam, NeVnam,
+                         SIInam1, SIInam2, H13nam, H12nam, H11nam, H10nam, H9nam, H8nam, HeInam,
+                         CAII_Knam, CAII_Hnam, Gbandnam])
+
+    line_wav = np.array([CIVwav2, CIVwav2, HeIIwav0, OIIIwav01, OIIIwav02, SiIIIwav1, SiIIIwav2,
+                         CIIIwav1, CIIIwav2, Lyalphawav, OIIwav1, OIIwav2, OIIIwav1, OIIIwav2,
+                         OIIIwav3, Halphawav, Hbetawav, Hdeltawav, Hgammawav, NeIIIwav, NeVwav,
+                         SIIwav1,SIIwav2, H13wav, H12wav, H11wav, H10wav, H9wav, H8wav, HeIwav,
+                         CaII_Kwav, CaII_Hwav, Gbandwav])
+
+    return line_names, line_wav
